<?php
namespace EMS\PortalFuncionario\Geral\Repository;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Util\DateUtil;

class ComunicadoRepository {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getComunicados(
        string $nrvinculom,
        string $nrorg,
        string $dtmescompetenc = null,
        string $data = null
    ) {
        $dtmescompetenc = empty($dtmescompetenc) ? DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO) : null;
        $sysdate = empty($data) ? date("d/m/Y") : null;

        $sql = <<<SQL
            SELECT
                COM.DSTITULOCOMUNICADO,
                COM.DSCOMUNICADO,
                COM.NRCOMUNICADO,
                TO_CHAR(COM.DTINIVIGENCIA, 'DD/MM/YYYY') AS DTINIVIGENCIA,
                TO_CHAR(COM.DTFIMVIGENCIA, 'DD/MM/YYYY') AS DTFIMVIGENCIA
            FROM GPE_COMUNICADO COM
            JOIN GPE_VINCULOH VH ON VH.NRORG      = COM.NRORG
                                AND VH.NRVINCULOM = :P_NRVINCULOM
                                AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                        WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                                                            AND NRORG      = VH.NRORG
                                                            AND NRVINCULOM = VH.NRVINCULOM )
            WHERE COM.NRORG    = :P_NRORG
                AND COM.DTINIVIGENCIA <= TO_DATE(:P_SYSDATE, 'DD/MM/YYYY')
                AND COM.DTFIMVIGENCIA >= TO_DATE(:P_SYSDATE, 'DD/MM/YYYY')
                AND (
                       INSTR(',' || COM.NRESTRUTLEGAL || ',', ',' || VH.NRESTRUTLEGAL || ',') <> 0
                    OR INSTR(',' || COM.NRESTRUTSIND || ',', ',' || VH.NRESTRUTSIND || ',') <> 0
                    OR INSTR(',' || COM.NRESTRUTGEREN || ',', ',' || VH.NRESTRUTGEREN || ',') <> 0
                    OR INSTR(',' || COM.NRVINCULOM || ',', ',' || :P_NRVINCULOM || ',') <> 0 )
SQL;

        $params = [
            "P_NRORG"                       => $nrorg,
            "P_NRVINCULOM"                  => $nrvinculom,
            "P_DTMESCOMPETENC"              => $dtmescompetenc,
            "P_SYSDATE"                     => $sysdate
        ];

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        $comunicados = $std->fetchAll();

        return $comunicados;
    }

}