<?php
namespace EMS\PortalFuncionario\Geral\Repository;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use EMS\PortalFuncionario\Util\NovoCodigo;
use EMS\PortalFuncionario\Geral\Repository\ApuracaoRepository;

class OcorrenciaRepository extends Crud {
    /** @var EntityManager */
    protected $entityManager;

    /** @var ApuracaoRepository */
    protected $apuracaoRepository;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager,
        ApuracaoRepository $apuracaoRepository
    ) {
        $this->entityManager        = $entityManager;
        $this->connection           = $entityManager->getConnection();
        $this->apuracaoRepository   = $apuracaoRepository;
    }

    public function getTipoProgocorrencia(string $nrorg, string $nrtipoprogocorrencia) : string {
        try {
            $sql = <<<SQL
                SELECT *
                FROM PTO_TIPOPROGOCORRENCIA TP
                WHERE (TP.NRORG = :P_NRORG OR TP.NRORG = :P_NRORG_PADRAO)
                            AND TP.NRTIPOPROGOCORRENCIA = :P_NRTIPOPROGOCORRENCIA
            ORDER BY TP.NMTIPOPROGOCORRENCIA
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRTIPOPROGOCORRENCIA"    => $nrtipoprogocorrencia,
                    "P_NRORG_PADRAO"            => 0
                )
            );

            $result = $query->fetchAll();

            $idprogocorconfirmada = '';
            if ($result[0]['IDPROGOCORACONFIRMAR'] == "S") {
                $idprogocorconfirmada = "P";
            } else {
                $idprogocorconfirmada = "S";
            }
            return $idprogocorconfirmada;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraOcorrenciaFrequencia(string $nrvinculom, string $dtinicioprogocor, string $dtfimprogocor, string $iddiassemana, string $idtipocalendario, string $idtipohorario, string $nrhordiam, string $nrintervalohorario, string $nrmotivoocorfreq, string $qtmaximaprogocor, string $dsobservacao, string $nrtipoprogocorrencia, string $nrmarcacaoponto, string $idprogocorconfirmada, string $dsobservacaorescusa, string $hrprojetarintervalonprog, string $dtinclusao, string $cdoperinclusao, string $nrorg) : int {
        try {
            $nrprogocorrencia = NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 22, false);

            $sql = <<<SQL
                INSERT INTO PTO_PROGOCORRENCIA (
                    NRPROGOCORRENCIA,
                    NRVINCULOM,
                    DTINICIOPROGOCOR,
                    DTFIMPROGOCOR,
                    IDDIASSEMANA,
                    IDTIPODIACALENDARIO,
                    IDTIPOHORARIO,
                    NRHORDIAM,
                    NRINTERVALOHORARIO,
                    NRMOTIVOOCORFREQ,
                    QTMAXIMAPROGOCOR,
                    DSOBSERVACAO,
                    NRTIPOPROGOCORRENCIA,
                    NRMARCACAOPONTO,
                    IDPROGOCORCONFIRMADA,
                    DSOBSERVACAORECUSA,
                    HRPROJETARINTERVALONPROG,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORG
                ) VALUES (
                    :P_NRPROGOCORRENCIA,
                    :P_NRVINCULOM,
                    TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                    TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                    :P_IDDIASSEMANA,
                    :P_IDTIPOCALENDARIO,
                    :P_IDTIPOHORARIO,
                    :P_NRHORDIAM,
                    :P_NRINTERVALOHORARIO,
                    :P_NRMOTIVOOCORFREQ,
                    :P_QTMAXIMAPROGOCOR,
                    :P_DSOBSERVACAO,
                    :P_NRTIPOPROGOCORRENCIA,
                    :P_NRMARCACAOPONTO,
                    :P_IDPROGOCORCONFIRMADA,
                    :P_DSOBSERVACAORESCUSA,
                    :P_HRPROJETARINTERVALONPROG,
                    TO_DATE(:P_DTINCLUSAO, 'DD/MM/YYYY'),
                    :P_CDOPERINCLUSAO,
                    :P_NRORG
                )
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA"            => $nrprogocorrencia,
                    "P_NRVINCULOM"                  => $nrvinculom,
                    "P_DTINICIOPROGOCOR"            => $dtinicioprogocor,
                    "P_DTFIMPROGOCOR"               => $dtfimprogocor,
                    "P_IDDIASSEMANA"                => $iddiassemana,
                    "P_IDTIPOCALENDARIO"            => $idtipocalendario,
                    "P_IDTIPOHORARIO"               => $idtipohorario,
                    "P_NRHORDIAM"                   => $nrhordiam,
                    "P_NRINTERVALOHORARIO"          => $nrintervalohorario,
                    "P_NRMOTIVOOCORFREQ"            => $nrmotivoocorfreq,
                    "P_QTMAXIMAPROGOCOR"            => $qtmaximaprogocor,
                    "P_DSOBSERVACAO"                => $dsobservacao,
                    "P_NRTIPOPROGOCORRENCIA"        => $nrtipoprogocorrencia,
                    "P_NRMARCACAOPONTO"             => $nrmarcacaoponto,
                    "P_IDPROGOCORCONFIRMADA"        => $idprogocorconfirmada,
                    "P_DSOBSERVACAORESCUSA"         => $dsobservacaorescusa,
                    "P_HRPROJETARINTERVALONPROG"    => $hrprojetarintervalonprog,
                    "P_DTINCLUSAO"                  => $dtinclusao,
                    "P_CDOPERINCLUSAO"              => $cdoperinclusao,
                    "P_NRORG"                       => $nrorg
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeOcorrenciaFrequencia(string $nrvinculom, string $dtinicioprogocor, string $dtfimprogocor, string $nrtipoprogocorrencia, string $nrmarcacaoponto, string $nrprogocorrencia, string $nrorg) {
        try {
            if ($nrmarcacaoponto != '') {
                $condition = "AND NRMARCACAOPONTO = $nrmarcacaoponto";
            } elseif ($nrprogocorrencia != '') {
                $condition = "AND NRPROGOCORRENCIA = $nrprogocorrencia";
            } elseif ($nrtipoprogocorrencia != '') {
                $condition = "AND NRTIPOPROGOCORRENCIA = $nrtipoprogocorrencia";
            } else {
                $condition = "AND 1 = 1";
            }

            $sql = <<<SQL
                DELETE FROM
                    PTO_PROGOCORRENCIA
                WHERE NRORG                = $nrorg
                    AND NRVINCULOM           = $nrvinculom
                    AND DTINICIOPROGOCOR     = TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY')
                    AND DTFIMPROGOCOR        = TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY')
                    $condition
SQL;
            $query = $this->connection->prepare($sql);

            $result = $query->execute(
                array(
                    "P_DTINICIOPROGOCOR"    => $dtinicioprogocor,
                    "P_DTFIMPROGOCOR"       => $dtfimprogocor
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrencias(string $nrorg, string $nrvinculom, string $idprogocorconfirmada, string $nrperiodoapuracao) {
        try {
            if (empty($nrperiodoapuracao)) {
                $periodo = $this->apuracaoRepository->getPeriodoApuracaoAtual($nrvinculom, $nrorg);
            } else {
                $periodo = $this->apuracaoRepository->getPeriodo($nrorg, $nrperiodoapuracao);
            }

            if (empty($periodo)) {
                throw new \Exception('Período não encontrado.');
            }

            $dtinicialapuracao = $periodo['DTINICIALAPURACAO'];
            $dtfinalapuracao = $periodo['DTFINALAPURACAO'];

            $sql = <<<SQL
                SELECT
                    PO.IDPROGOCORCONFIRMADA,
                    TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') AS DTINICIOPROGOCOR,
                    MO.DSMOTIVOOCORFREQ,
                    SF.DSSITUFREQUENCIA
                FROM PTO_PROGOCORRENCIA PO
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = PO.NRORG AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
                LEFT JOIN PTO_SITUFREQUENCIA SF ON SF.NRORG = PO.NRORG AND SF.NRSITUFREQUENCIA = PO.NRSITUFREQUENCIA
                WHERE PO.NRORG = :P_NRORG
                    AND (PO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                    AND PO.IDPROGOCORCONFIRMADA = :P_IDPROGOCORCONFIRMADA /* somente ocorrência pendentes */
                    AND PO.DTINICIOPROGOCOR BETWEEN TO_DATE(:P_DTINICIALAPURACAO, 'DD/MM/YYYY') AND TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRVINCULOM"              => $nrvinculom,
                    "P_DTINICIALAPURACAO"       => $dtinicialapuracao,
                    "P_DTFINALAPURACAO"         => $dtfinalapuracao,
                    "P_IDPROGOCORCONFIRMADA"    => $idprogocorconfirmada
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasPeriodo(string $nrorg, string $nrvinculom, string $nrperiodoapuracao) {
        try {
            if (empty($nrperiodoapuracao)) {
                $periodo = $this->apuracaoRepository->getPeriodoApuracaoAtual($nrvinculom, $nrorg);
            } else {
                $periodo = $this->apuracaoRepository->getPeriodo($nrorg, $nrperiodoapuracao);
            }

            if (empty($periodo)) {
                throw new \Exception('Período não encontrado.');
            }

            $dtinicialapuracao = $periodo['DTINICIALAPURACAO'];
            $dtfinalapuracao = $periodo['DTFINALAPURACAO'];

            $sql = <<<SQL
                SELECT
                    PO.NRORG,
                    PO.NRPROGOCORRENCIA,
                    PO.NRVINCULOM,
                    TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') AS DTINICIOPROGOCOR,
                    TO_CHAR(PO.DTFIMPROGOCOR, 'DD/MM/YYYY') AS DTFIMPROGOCOR,
                    PO.IDDIASSEMANA,
                    PO.IDTIPODIACALENDARIO,
                    PO.IDTIPOHORARIO,
                    PO.NRMOTIVOOCORFREQ,
                    PO.NRHORDIAM,
                    PO.NRSITUFREQUENCIA,
                    PO.NRSITUFREQGATILHO,
                    PO.QTMINGATILHOPROGOCOR,
                    PO.QTMAXGATILHOPROGOCOR,
                    PO.VRCOEFICIENTEACRESCIMO,
                    PO.IDPROGOCORRENCIA,
                    PO.QTMINIMAPROGOCOR,
                    PO.QTMAXIMAPROGOCOR,
                    PO.DSOBSERVACAO,
                    PO.IDATIVO,
                    PO.NRORGINCLUSAO,
                    PO.DTINCLUSAO,
                    PO.CDOPERINCLUSAO,
                    PO.NRORGULTATU,
                    PO.DTULTATU,
                    PO.CDOPERULTATU,
                    PO.NRTIPOPROGOCORRENCIA,
                    PO.NRMARCACAOPONTO,
                    PO.NRINTERVALOHORARIO,
                    PO.IDPROGOCORCONFIRMADA,
                    PO.CDOPERVERIFICADO,
                    PO.DSOBSERVACAORECUSA,
                    PO.HRPROJETARINTERVALONPROG,
                    MO.DSMOTIVOOCORFREQ,
                    SF.DSSITUFREQUENCIA,
                    DECODE (PO.IDPROGOCORCONFIRMADA, 'S', 'Confirmada',
                                     'N', 'Rejeitada',
                                     'P', 'Pendente',
                                     'Pendente') DSCONFIRM
                FROM PTO_PROGOCORRENCIA PO
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = PO.NRORG AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
                LEFT JOIN PTO_SITUFREQUENCIA SF ON SF.NRORG = PO.NRORG AND SF.NRSITUFREQUENCIA = PO.NRSITUFREQUENCIA
                WHERE PO.NRORG = :P_NRORG
                    AND (PO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                    AND PO.DTINICIOPROGOCOR BETWEEN TO_DATE(:P_DTINICIALAPURACAO, 'DD/MM/YYYY') AND TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRVINCULOM"              => $nrvinculom,
                    "P_DTINICIALAPURACAO"       => $dtinicialapuracao,
                    "P_DTFINALAPURACAO"         => $dtfinalapuracao
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function getMotivoOcorFreq(string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT DSMOTIVOOCORFREQ,
                       NRMOTIVOOCORFREQ
                FROM PTO_MOTIVOOCORFREQ
                WHERE NRORG = :P_NRORG
                ORDER BY DSMOTIVOOCORFREQ
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function newOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];
            $nrprogocorrencia = NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 22, false);

            $sql = <<<SQL
                INSERT INTO PTO_PROGOCORRENCIA (
                    NRPROGOCORRENCIA,
                    NRVINCULOM,
                    DTINICIOPROGOCOR,
                    DTFIMPROGOCOR,
                    IDTIPOHORARIO,
                    NRMOTIVOOCORFREQ,
                    DSOBSERVACAO,
                    NRTIPOPROGOCORRENCIA,
                    IDPROGOCORCONFIRMADA,
                    NRORG
                ) VALUES (
                    :P_NRPROGOCORRENCIA,
                    :P_NRVINCULOM,
                    TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                    TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                    :P_IDTIPOHORARIO,
                    :P_NRMOTIVOOCORFREQ,
                    :P_DSOBSERVACAO,
                    :P_NRTIPOPROGOCORRENCIA,
                    :P_IDPROGOCORCONFIRMADA,
                    :P_NRORG
                )
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA"            => $nrprogocorrencia,
                    "P_NRVINCULOM"                  => isset($row['NRVINCULOM'])            ? $row['NRVINCULOM'] : NULL ,
                    "P_DTINICIOPROGOCOR"            => isset($row['DTINICIOPROGOCOR'])      ? $row['DTINICIOPROGOCOR'] : NULL ,
                    "P_DTFIMPROGOCOR"               => isset($row['DTFIMPROGOCOR'])         ? $row['DTFIMPROGOCOR'] : NULL ,
                    "P_IDTIPOHORARIO"               => isset($row['IDTIPOHORARIO'])         ? $row['IDTIPOHORARIO'] : 'N' ,
                    "P_NRMOTIVOOCORFREQ"            => isset($row['NRMOTIVOOCORFREQ'])      ? $row['NRMOTIVOOCORFREQ'] : NULL ,
                    "P_DSOBSERVACAO"                => isset($row['DSOBSERVACAO'])          ? $row['DSOBSERVACAO'] : NULL ,
                    "P_NRTIPOPROGOCORRENCIA"        => isset($row['NRTIPOPROGOCORRENCIA'])  ? $row['NRTIPOPROGOCORRENCIA'] : NULL ,
                    "P_IDPROGOCORCONFIRMADA"        => isset($row['IDPROGOCORCONFIRMADA'])  ? $row['IDPROGOCORCONFIRMADA'] : NULL ,
                    "P_NRORG"                       => $nrorg
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                UPDATE PTO_PROGOCORRENCIA
                    SET
                        DTINICIOPROGOCOR      =  TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                        DTFIMPROGOCOR         =  TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                        IDTIPOHORARIO         =  :P_IDTIPOHORARIO,
                        NRMOTIVOOCORFREQ      =  :P_NRMOTIVOOCORFREQ,
                        DSOBSERVACAO          =  :P_DSOBSERVACAO,
                        NRTIPOPROGOCORRENCIA  =  :P_NRTIPOPROGOCORRENCIA,
                        IDPROGOCORCONFIRMADA  =  :P_IDPROGOCORCONFIRMADA
                    WHERE
                        NRPROGOCORRENCIA = :P_NRPROGOCORRENCIA AND NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_DTINICIOPROGOCOR"            => isset($row['DTINICIOPROGOCOR'])      ? $row['DTINICIOPROGOCOR'] : NULL ,
                    "P_DTFIMPROGOCOR"               => isset($row['DTFIMPROGOCOR'])         ? $row['DTFIMPROGOCOR'] : NULL ,
                    "P_IDTIPOHORARIO"               => isset($row['IDTIPOHORARIO'])         ? $row['IDTIPOHORARIO'] : 'N' ,
                    "P_NRMOTIVOOCORFREQ"            => isset($row['NRMOTIVOOCORFREQ'])      ? $row['NRMOTIVOOCORFREQ'] : NULL ,
                    "P_DSOBSERVACAO"                => isset($row['DSOBSERVACAO'])          ? $row['DSOBSERVACAO'] : NULL ,
                    "P_NRTIPOPROGOCORRENCIA"        => isset($row['NRTIPOPROGOCORRENCIA'])  ? $row['NRTIPOPROGOCORRENCIA'] : NULL ,
                    "P_IDPROGOCORCONFIRMADA"        => isset($row['IDPROGOCORCONFIRMADA'])  ? $row['IDPROGOCORCONFIRMADA'] : NULL ,
                    "P_NRPROGOCORRENCIA"            => isset($row['NRPROGOCORRENCIA'])      ? $row['NRPROGOCORRENCIA'] : NULL ,
                    "P_NRORG"                       => $nrorg
                )
            );
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                DELETE FROM PTO_PROGOCORRENCIA
                    WHERE
                    NRPROGOCORRENCIA = :P_NRPROGOCORRENCIA AND NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA" => isset($row['NRPROGOCORRENCIA'])      ? $row['NRPROGOCORRENCIA'] : NULL ,
                    "P_NRORG"            => $nrorg
                )
            );
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasPorTipoOcorrencia(string $nrvinculom, string $nrorg, string $nrtipoprogocorrencia, string $dtinicioprogocor = null): array {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    PTO_PROGOCORRENCIA OCOR
                WHERE
                    OCOR.NRORG = :P_NRORG
                    AND OCOR.NRTIPOPROGOCORRENCIA = :P_NRTIPOPROGOCORRENCIA
                    AND OCOR.NRVINCULOM = :P_NRVINCULOM
                    AND (OCOR.DTINICIOPROGOCOR = TO_DATE(:P_DTINICIOPROGOCOR,'DD/MM/YYYY') OR :P_DTINICIOPROGOCOR IS NULL)
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    'P_NRORG'                   => $nrorg,
                    'P_NRTIPOPROGOCORRENCIA'    => $nrtipoprogocorrencia,
                    'P_NRVINCULOM'              => $nrvinculom,
                    'P_DTINICIOPROGOCOR'        => $dtinicioprogocor
                )
            );

            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMotivosOcorrencia(array $params): array
    {
        $sql = <<<SQL
            SELECT
              NRMOTIVOOCORFREQ,
              DSMOTIVOOCORFREQ,
              IDTIPOINTERVALO
            FROM PTO_MOTIVOOCORFREQ
            WHERE (NRORG = 0 OR NRORG = :P_NRORG)
              AND IDATIVO = 'S'
            ORDER BY DSMOTIVOOCORFREQ
SQL;

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function getOcorrenciasDia(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                OCOR.*,
                TIPO.NMTIPOPROGOCORRENCIA,
                MOT.DSMOTIVOOCORFREQ
            FROM
                PTO_PROGOCORRENCIA OCOR
                LEFT JOIN
                PTO_TIPOPROGOCORRENCIA TIPO ON
                    TIPO.NRORG IN (OCOR.NRORG, :P_NRORG_PADRAO) AND
                    TIPO.NRTIPOPROGOCORRENCIA = OCOR.NRTIPOPROGOCORRENCIA
                LEFT JOIN PTO_MOTIVOOCORFREQ MOT ON
                    MOT.NRORG IN (OCOR.NRORG, :P_NRORG_PADRAO) AND
                    MOT.NRMOTIVOOCORFREQ = OCOR.NRMOTIVOOCORFREQ
            WHERE
                OCOR.NRORG = :P_NRORG AND
                OCOR.NRVINCULOM = :P_NRVINCULO AND
                TO_DATE(:P_DATE, 'DD/MM/YYYY') BETWEEN OCOR.DTINICIOPROGOCOR AND OCOR.DTFIMPROGOCOR AND
                OCOR.NRTIPOPROGOCORRENCIA NOT IN (20)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $ocorrencias = $std->fetchAll();

        return $ocorrencias;
    }
}